import type { IUnleashConfig } from '../../types/option';
import type { IUnleashStores } from '../../types/stores';
import type VersionService from '../../services/version-service';
import type { GetActiveUsers } from './getActiveUsers';
import type { ProjectModeCount } from '../project/project-store';
import type { GetProductionChanges } from './getProductionChanges';
export type TimeRange = 'allTime' | '30d' | '7d';
export interface InstanceStats {
    instanceId: string;
    timestamp: Date;
    versionOSS: string;
    versionEnterprise?: string;
    users: number;
    serviceAccounts: number;
    apiTokens: Map<string, number>;
    featureToggles: number;
    archivedFeatureToggles: number;
    projects: ProjectModeCount[];
    contextFields: number;
    roles: number;
    customRootRoles: number;
    customRootRolesInUse: number;
    featureExports: number;
    featureImports: number;
    groups: number;
    environments: number;
    segments: number;
    strategies: number;
    SAMLenabled: boolean;
    OIDCenabled: boolean;
    clientApps: {
        range: TimeRange;
        count: number;
    }[];
    activeUsers: Awaited<ReturnType<GetActiveUsers>>;
    productionChanges: Awaited<ReturnType<GetProductionChanges>>;
    previousDayMetricsBucketsCount: {
        enabledCount: number;
        variantCount: number;
    };
    maxEnvironmentStrategies: number;
    maxConstraints: number;
    maxConstraintValues: number;
}
export type InstanceStatsSigned = Omit<InstanceStats, 'projects'> & {
    projects: number;
    sum: string;
};
export declare class InstanceStatsService {
    private logger;
    private strategyStore;
    private userStore;
    private featureToggleStore;
    private contextFieldStore;
    private projectStore;
    private groupStore;
    private environmentStore;
    private segmentStore;
    private roleStore;
    private eventStore;
    private apiTokenStore;
    private versionService;
    private settingStore;
    private clientInstanceStore;
    private clientMetricsStore;
    private flagResolver;
    private appCount?;
    getActiveUsers: GetActiveUsers;
    getProductionChanges: GetProductionChanges;
    private featureStrategiesReadModel;
    private trafficDataUsageStore;
    constructor({ featureToggleStore, userStore, projectStore, environmentStore, strategyStore, contextFieldStore, groupStore, segmentStore, roleStore, settingStore, clientInstanceStore, eventStore, apiTokenStore, clientMetricsStoreV2, featureStrategiesReadModel, trafficDataUsageStore, }: Pick<IUnleashStores, 'featureToggleStore' | 'userStore' | 'projectStore' | 'environmentStore' | 'strategyStore' | 'contextFieldStore' | 'groupStore' | 'segmentStore' | 'roleStore' | 'settingStore' | 'clientInstanceStore' | 'eventStore' | 'apiTokenStore' | 'clientMetricsStoreV2' | 'featureStrategiesReadModel' | 'trafficDataUsageStore'>, { getLogger, flagResolver, }: Pick<IUnleashConfig, 'getLogger' | 'flagResolver'>, versionService: VersionService, getActiveUsers: GetActiveUsers, getProductionChanges: GetProductionChanges);
    getProjectModeCount(): Promise<ProjectModeCount[]>;
    getToggleCount(): Promise<number>;
    getArchivedToggleCount(): Promise<number>;
    hasOIDC(): Promise<boolean>;
    hasSAML(): Promise<boolean>;
    getStats(): Promise<InstanceStats>;
    groupCount(): Promise<number>;
    roleCount(): Promise<number>;
    customRolesCount(): Promise<number>;
    customRolesCountInUse(): Promise<number>;
    segmentCount(): Promise<number>;
    contextFieldCount(): Promise<number>;
    strategiesCount(): Promise<number>;
    environmentCount(): Promise<number>;
    countPreviousDayHourlyMetricsBuckets(): Promise<{
        enabledCount: number;
        variantCount: number;
    }>;
    countApiTokensByType(): Promise<Map<string, number>>;
    getRegisteredUsers(): Promise<number>;
    countServiceAccounts(): Promise<number>;
    getCurrentTrafficData(): Promise<number>;
    getLabeledAppCounts(): Promise<Partial<{
        [key in TimeRange]: number;
    }>>;
    getAppCountSnapshot(range: TimeRange): number | undefined;
    getSignedStats(): Promise<InstanceStatsSigned>;
}
//# sourceMappingURL=instance-stats-service.d.ts.map