import type { FromSchema } from 'json-schema-to-ts';
export declare const aiChatSchema: {
    readonly $id: "#/components/schemas/aiChatSchema";
    readonly type: "object";
    readonly description: "Describes an Unleash AI chat.";
    readonly additionalProperties: false;
    readonly required: readonly ["id", "userId", "createdAt", "messages"];
    readonly properties: {
        readonly id: {
            readonly type: "string";
            readonly pattern: "^[0-9]+$";
            readonly description: "The chat's ID. Chat IDs are incrementing integers. In other words, a more recently created chat will always have a higher ID than an older one. This ID is represented as a string since it is a BigInt.";
            readonly example: "7";
        };
        readonly userId: {
            readonly type: "integer";
            readonly description: "The ID of the user that the chat belongs to.";
            readonly example: 7;
        };
        readonly createdAt: {
            readonly type: "string";
            readonly format: "date-time";
            readonly description: "The date and time of when the chat was created.";
            readonly example: "2023-12-27T13:37:00+01:00";
        };
        readonly messages: {
            readonly type: "array";
            readonly description: "The messages exchanged between the user and the Unleash AI.";
            readonly items: {
                readonly $ref: "#/components/schemas/aiChatMessageSchema";
            };
        };
    };
    readonly components: {
        readonly schemas: {
            readonly aiChatMessageSchema: {
                readonly $id: "#/components/schemas/aiChatMessageSchema";
                readonly type: "object";
                readonly description: "Describes an Unleash AI chat message.";
                readonly additionalProperties: false;
                readonly required: readonly ["role", "content"];
                readonly properties: {
                    readonly role: {
                        readonly type: "string";
                        readonly enum: readonly ["system", "user", "assistant"];
                        readonly description: "The role of the message sender.";
                        readonly example: "user";
                    };
                    readonly content: {
                        readonly type: "string";
                        readonly description: "The message content.";
                        readonly example: "What is your purpose?";
                    };
                };
                readonly components: {
                    readonly schemas: {};
                };
            };
        };
    };
};
export type AIChatSchema = FromSchema<typeof aiChatSchema>;
//# sourceMappingURL=ai-chat-schema.d.ts.map